import uuid
from importlib import import_module
import os
import django.db.utils
from django.apps import apps
from django.conf import settings
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sessions.models import Session
from django.db.migrations import Migration
from django.db.migrations.recorder import MigrationRecorder

from activitylog.models import CRUDEvent, LoginEvent, RequestEvent, CorsEvent


def get_model_list(class_list):
    """Get a list of model classes from a list of strings.

    Receives a list of strings with app_name.model_name format and turns them into classes.
    If an item is already a class, it ignores it.
    """
    for idx, item in enumerate(class_list):
        if isinstance(item, (str,)):
            model_class = apps.get_model(item)
            class_list[idx] = model_class


# Should Django Activity log model/auth/request events?
WATCH_AUTH_EVENTS = getattr(settings, "DJANGO_ACTIVITY_LOG_WATCH_AUTH_EVENTS", True)
WATCH_MODEL_EVENTS = getattr(settings, "DJANGO_ACTIVITY_LOG_WATCH_MODEL_EVENTS", True)
WATCH_REQUEST_EVENTS = getattr(settings, "DJANGO_ACTIVITY_LOG_WATCH_REQUEST_EVENTS", True)
WATCH_CORS_EVENTS = getattr(settings, "DJANGO_ACTIVITY_LOG_WATCH_CORS_EVENTS", True)

X_FORWARDED_FOR = getattr(settings, "DJANGO_ACTIVITY_LOG_REMOTE_ADDR_HEADER", 'HTTP_X_FORWARDED_FOR')
if X_FORWARDED_FOR:
    REMOTE_ADDR_HEADER = X_FORWARDED_FOR.split(',')[0].strip()
else:
    REMOTE_ADDR_HEADER = getattr(settings, "DJANGO_ACTIVITY_LOG_REMOTE_ADDR_HEADER", "REMOTE_ADDR")

HTTP_SEC_CH_UA = getattr(settings, "DJANGO_ACTIVITY_LOG_BROWSER", "HTTP_SEC_CH_UA")
HTTP_SEC_CH_UA_PLATFORM = getattr(settings, "DJANGO_ACTIVITY_LOG_PLATFORM", "HTTP_SEC_CH_UA_PLATFORM")
GNOME_SHELL_SESSION_MODE = getattr(settings, "DJANGO_ACTIVITY_LOG_OPERATING_SYSTEM", "GNOME_SHELL_SESSION_MODE")

USER_DB_CONSTRAINT = bool(getattr(settings, "DJANGO_ACTIVITY_LOG_USER_DB_CONSTRAINT", True))

# logging backend settings
LOGGING_BACKEND = getattr(
    settings, "DJANGO_ACTIVITY_LOG_LOGGING_BACKEND", "activitylog.backends.ModelBackend"
)

# Models which Django Activity Log will not log.
# By default, all but some models will be audited.
# The list of excluded models can be overwritten or extended
# by defining the following settings in the project.
UNREGISTERED_CLASSES = [
    CRUDEvent,
    LoginEvent,
    RequestEvent,
    CorsEvent,
    Migration,
    Session,
    Permission,
    ContentType,
    MigrationRecorder.Migration,
]

# Import and unregister LogEntry class only if Django Admin app is installed
if apps.is_installed("django.contrib.admin"):
    from django.contrib.admin.models import LogEntry

    UNREGISTERED_CLASSES += [LogEntry]

UNREGISTERED_CLASSES = getattr(
    settings, "DJANGO_ACTIVITY_LOG_UNREGISTERED_CLASSES_DEFAULT", UNREGISTERED_CLASSES
)
UNREGISTERED_CLASSES.extend(
    getattr(settings, "DJANGO_ACTIVITY_LOG_UNREGISTERED_CLASSES_EXTRA", [])
)
get_model_list(UNREGISTERED_CLASSES)

# Models which Django Activity Log WILL log.
# If the following setting is defined in the project,
# only the listed models will be audited, and every other
# model will be excluded.
REGISTERED_CLASSES = getattr(settings, "DJANGO_ACTIVITY_LOG_REGISTERED_CLASSES", [])
get_model_list(REGISTERED_CLASSES)

# URLs which Django Activity Log will not log.
# By default, all but some URL requests will be logged.
# The list of excluded URLs can be overwritten or extended
# by defining the following settings in the project.
# Note: it is a list of regular expressions.
UNREGISTERED_URLS = [r"^/admin/", r"^/static/", r"^/favicon.ico$"]
UNREGISTERED_URLS = getattr(
    settings, "DJANGO_ACTIVITY_LOG_UNREGISTERED_URLS_DEFAULT", UNREGISTERED_URLS
)
UNREGISTERED_URLS.extend(getattr(settings, "DJANGO_ACTIVITY_LOG_UNREGISTERED_URLS_EXTRA", []))

# URLs which Django Activity Log WILL log.
# If the following setting is defined in the project,
# only the listed URLs will be audited, and every other
# URL will be excluded.
REGISTERED_URLS = getattr(settings, "DJANGO_ACTIVITY_LOG_REGISTERED_URLS", [])

# By default all modules are listed in the admin.
# This can be changed with the following settings.
ADMIN_SHOW_MODEL_EVENTS = getattr(
    settings, "DJANGO_ACTIVITY_LOG_ADMIN_SHOW_MODEL_EVENTS", True
)
ADMIN_SHOW_AUTH_EVENTS = getattr(settings, "DJANGO_ACTIVITY_LOG_ADMIN_SHOW_AUTH_EVENTS", True)

ADMIN_SHOW_REQUEST_EVENTS = getattr(
    settings, "DJANGO_ACTIVITY_LOG_ADMIN_SHOW_REQUEST_EVENTS", True
)

ADMIN_SHOW_CORS_EVENTS = getattr(
    settings, "DJANGO_ACTIVITY_LOG_ADMIN_SHOW_CORS_EVENTS", True
)

# project defined callbacks
CRUD_DIFFERENCE_CALLBACKS = []
CRUD_DIFFERENCE_CALLBACKS = getattr(
    settings, "DJANGO_ACTIVITY_LOG_CRUD_DIFFERENCE_CALLBACKS", CRUD_DIFFERENCE_CALLBACKS
)
DATABASE_ALIAS = getattr(
    settings, "DJANGO_ACTIVITY_LOG_DATABASE_ALIAS", django.db.utils.DEFAULT_DB_ALIAS
)
# The callbacks could come in as an iterable of strings, where each string is the
# package.module.function
for idx, callback in enumerate(CRUD_DIFFERENCE_CALLBACKS):
    if not callable(callback):  # keep as is if it is callable
        CRUD_DIFFERENCE_CALLBACKS[idx] = getattr(
            import_module(".".join(callback.split(".")[:-1])),
            callback.split(".")[-1],
            None,
        )

"""although this setting "exists" here we do not intend to use it anywhere due to test run
issues maybe we can properly solve this at a latter time. instead, anything inside of this
library should do the same getattr check here, based on normal `settings` from
`django.conf`."""
CRUD_EVENT_NO_CHANGED_FIELDS_SKIP = getattr(
    settings, "DJANGO_ACTIVITY_LOG_CRUD_EVENT_NO_CHANGED_FIELDS_SKIP", False
)

"""Purge table optimization:
If TRUNCATE_TABLE_SQL_STATEMENT is not empty, we use it as custom sql statement to speed up
table truncation bypassing ORM, i.e.:

  DJANGO_ACTIVITY_LOG_TRUNCATE_TABLE_SQL_STATEMENT = 'TRUNCATE TABLE "{db_table}"'

Else we use Django Orm as follows:

  model.objects.all().delete()

which is however much costly when many rows are involved"""
TRUNCATE_TABLE_SQL_STATEMENT = getattr(
    settings, "DJANGO_ACTIVITY_LOG_TRUNCATE_TABLE_SQL_STATEMENT", ""
)

# Changeview filters configuration
CRUD_EVENT_LIST_FILTER = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_CRUD_EVENT_LIST_FILTER",
    [
        "event_type",
        "content_type",
        "user",
        "datetime",
    ],
)
LOGIN_EVENT_LIST_FILTER = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_LOGIN_EVENT_LIST_FILTER",
    [
        "login_type",
        "user",
        "datetime",
    ],
)
REQUEST_EVENT_LIST_FILTER = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_REQUEST_EVENT_LIST_FILTER",
    [
        "method",
        "user",
        "datetime",
    ],
)

CORS_EVENT_LIST_FILTER = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_CORS_EVENT_LIST_FILTER",
    [
        "method",
        "user",
        "datetime",
    ],
)

# Search fields configuration
CRUD_EVENT_SEARCH_FIELDS = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_CRUD_EVENT_SEARCH_FIELDS",
    [
        "=object_id",
        "object_json_repr",
    ],
)
LOGIN_EVENT_SEARCH_FIELDS = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_LOGIN_EVENT_SEARCH_FIELDS",
    [
        "=remote_ip",
        "username",
    ],
)

REQUEST_EVENT_SEARCH_FIELDS = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_REQUEST_EVENT_SEARCH_FIELDS",
    [
        "=remote_ip",
        "user__username",
        "url",
        "query_string",
    ],
)

CORS_EVENT_SEARCH_FIELDS = getattr(
    settings,
    "DJANGO_ACTIVITY_LOG_CORS_EVENT_SEARCH_FIELDS",
    [
        "=remote_ip",
        "user__username",
        "url",
        "query_string",
    ],
)

READONLY_EVENTS = getattr(settings, "DJANGO_ACTIVITY_LOG_READONLY_EVENTS", False)

GEOIP_PATH = os.path.join('local/GeoLite2-City.mmdb')



from django.apps import AppConfig


class ActivitylogConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'activitylog'

    def ready(self):
        from activitylog.signals import (
            auth_signals,
            model_signals,
            request_signals,
            cors_signals,
        )


from urllib.parse import urlencode

from django.contrib import admin
from django.contrib.admin.templatetags.admin_urls import add_preserved_filters
from django.contrib.contenttypes.models import ContentType
from django.db.models import Model
from django.http import HttpRequest, HttpResponseRedirect
from django.shortcuts import redirect
from django.urls import path, reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _


class BaseProcessActionsAdminMixin:
    def get_action_methods(self):
        return {}

    def _get_path_info(self):
        return self.model._meta.app_label, self.model._meta.model_name

    def get_redirect_url__to_referer(self, request: HttpRequest):
        preserved_filters = self.get_preserved_filters(request)
        opts = self.model._meta

        return add_preserved_filters(
            {"preserved_filters": preserved_filters, "opts": opts},
            request.META.get("HTTP_REFERER", "/"),
        )

    def process_action(
        self,
        request,
        obj_id,
        action_key,
        **kwargs,
    ):
        action_methods = self.get_action_methods()
        action = action_methods[action_key]
        obj = self.get_object(request, obj_id)
        return action(request, obj)


class CRUDHistoryAdminMixin(BaseProcessActionsAdminMixin, admin.ModelAdmin):
    CRUD_HISTORY = "crud_history"
    crud_history_translated_title = _("CRUD history")

    def get_urls(self) -> list:
        urls = super().get_urls()
        info = self._get_path_info()
        crud_history_urls = [
            path(
                f"<path:object_id>/{self.CRUD_HISTORY}/",
                self.admin_site.admin_view(self.crud_history_view),
                name=f"%s_%s_{self.CRUD_HISTORY}" % info,
            )
        ]
        return crud_history_urls + urls

    def get_action_methods(self) -> dict:
        methods = super().get_action_methods()
        methods.update({self.CRUD_HISTORY: self.crud_history_action})
        return methods

    def crud_history_view(self, request: HttpRequest, object_id: int):
        return self.process_action(request, object_id, self.CRUD_HISTORY)

    def crud_history_action(self, request: HttpRequest, obj: Model) -> HttpResponseRedirect:
        base_history_url = reverse(
            "admin:activitylog_crudevent_changelist",
        )
        app_label, model_name = self._get_path_info()
        content_type = ContentType.objects.get_by_natural_key(app_label, model_name)
        params = {"content_type__id": content_type.id, "object_id": obj.id}
        params = urlencode(params)
        history_url = f"{base_history_url}?{params}"

        return redirect(history_url)

    crud_history_action.short_description = crud_history_translated_title

    def get_crud_history_url(self, obj: Model) -> str:
        info = self._get_path_info()
        return reverse(f"admin:%s_%s_{self.CRUD_HISTORY}" % info, args=[obj.pk])

    def crud_history_link(self, obj: Model) -> str:
        crud_history_url = self.get_crud_history_url(obj=obj)
        crud_history_a = (
            f"<a href={crud_history_url}>> {self.crud_history_translated_title}</a>"
        )
        return format_html(crud_history_a)

    crud_history_link.allow_tags = True
    crud_history_link.short_description = crud_history_translated_title


# Example
class SomeModelAdmin(CRUDHistoryAdminMixin):
    list_display = (
        "id",
        "crud_history_link",
    )


import logging
from activitylog.models import CRUDEvent, LoginEvent, RequestEvent, CorsEvent

logger = logging.getLogger(__name__)


class ModelBackend:
    def request(self, request_info):
        return RequestEvent.objects.create(**request_info)

    def cors(self, cors_info):
        return CorsEvent.objects.create(**cors_info)

    def crud(self, crud_info):
        return CRUDEvent.objects.create(**crud_info)

    def login(self, login_info):
        return LoginEvent.objects.create(**login_info)



import json
from django.contrib import admin, messages
from django.contrib.auth import get_user_model
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import re_path, reverse
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from .settings import READONLY_EVENTS, TRUNCATE_TABLE_SQL_STATEMENT


def prettify_json(json_string):
    """Given a JSON string, it returns it as a safe formatted HTML."""
    escaped = escape(json_string)
    try:
        data = json.loads(escaped)
        html = json.dumps(data, sort_keys=True, indent=4)
    except Exception:
        html = escaped
    return html


class ActivityLogModelAdmin(admin.ModelAdmin):
    def get_changelist_instance(self, *args, **kwargs):
        changelist_instance = super().get_changelist_instance(*args, **kwargs)
        user_ids = [obj.user_id for obj in changelist_instance.result_list]
        self.users_by_id = {
            user.id: user for user in get_user_model().objects.filter(id__in=user_ids)
        }
        return changelist_instance

    def get_readonly_fields(self, request, obj=None):
        """Mark all fields of model as readonly if configured to do so."""
        if READONLY_EVENTS:
            return [f.name for f in self.model._meta.get_fields()]
        return self.readonly_fields

    @admin.display(description="User")
    def user_link(self, obj):
        user = self.users_by_id.get(obj.user_id)
        if user is None:
            return "-"
        escaped = escape(str(user))
        try:
            user_model = get_user_model()
            url = reverse(
                f"admin:{user_model._meta.app_label}_{user_model._meta.model_name}_change"
            )
            html = f'<a href="{url}">{escaped}</a>'
            html = f'<a href="{url}">{escaped}</a>'
        except Exception:
            html = escaped
        return mark_safe(html)  # noqa: S308

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        if READONLY_EVENTS:
            return False
        return super().has_delete_permission(request, obj)

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            re_path(
                r"^purge/$",
                self.admin_site.admin_view(self.purge),
                {},
                name=f"{self.model._meta.app_label}_{self.model._meta.model_name}_purge",
            ),
        ]
        return my_urls + urls

    def purge(self, request):
        return self.purge_objects(request)

    # Helper view to remove all rows in a table
    def purge_objects(self, request):
        """Remove all objects in this table.

        This action first displays a confirmation page; next, it deletes all objects and
        redirects back to the change list.
        """
        if READONLY_EVENTS:
            raise PermissionDenied

        def truncate_table(model):
            if TRUNCATE_TABLE_SQL_STATEMENT:
                from django.db import connection

                sql = TRUNCATE_TABLE_SQL_STATEMENT.format(db_table=model._meta.db_table)
                cursor = connection.cursor()
                cursor.execute(sql)
            else:
                model.objects.all().delete()

        modeladmin = self
        opts = modeladmin.model._meta

        # Check that the user has delete permission for the actual model
        if not request.user.is_superuser:
            raise PermissionDenied
        if not modeladmin.has_delete_permission(request):
            raise PermissionDenied

        # If the user has already confirmed or cancelled the deletion,
        # (eventually) do the deletion and return to the change list view again.
        if request.method == "POST":
            if "btn-confirm" in request.POST:
                try:
                    n = modeladmin.model.objects.count()
                    truncate_table(modeladmin.model)
                    modeladmin.message_user(
                        request, _("Successfully removed %d rows" % n), messages.SUCCESS
                    )
                except Exception as e:
                    modeladmin.message_user(
                        request, _("ERROR") + ": %r" % e, messages.ERROR
                    )
            else:
                modeladmin.message_user(
                    request, _("Action cancelled by user"), messages.SUCCESS
                )
            return HttpResponseRedirect(
                reverse(f"admin:{opts.app_label}_{opts.model_name}_changelist")
            )

        context = {
            "title": _("Purge all %s ... are you sure?") % opts.verbose_name_plural,
            "opts": opts,
            "app_label": opts.app_label,
        }

        # Display the confirmation page
        return render(request, "admin/activity/purge_confirmation.html", context)



import uuid
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _


def default_uuid():
    return getattr(settings, "DJANGO_ACTIVITY_LOG_PRIMARY_KEY", uuid.uuid4())


class CRUDEvent(models.Model):
    id = models.UUIDField(primary_key=True, default=default_uuid, editable=False, unique=True)
    CREATE = 1
    UPDATE = 2
    DELETE = 3
    M2M_CHANGE = 4
    M2M_CHANGE_REV = 5
    M2M_ADD = 6
    M2M_ADD_REV = 7
    M2M_REMOVE = 8
    M2M_REMOVE_REV = 9
    M2M_CLEAR = 10
    M2M_CLEAR_REV = 11

    TYPES = (
        (CREATE, _('Create')),
        (UPDATE, _('Update')),
        (DELETE, _('Delete')),
        (M2M_CHANGE, _('Many-to-Many Change')),
        (M2M_CHANGE_REV, _('Reverse Many-to-Many Change')),
        (M2M_ADD, _('Many-to-Many Add')),
        (M2M_ADD_REV, _('Reverse Many-to-Many Add')),
        (M2M_REMOVE, _('Many-to-Many Remove')),
        (M2M_REMOVE_REV, _('Reverse Many-to-Many Remove')),
        (M2M_CLEAR, _('Many-to-Many Clear')),
        (M2M_CLEAR_REV, _('Reverse Many-to-Many Clear')),
    )

    event_type = models.SmallIntegerField(choices=TYPES, verbose_name=_('Event type'))
    object_id = models.CharField(max_length=255, verbose_name=_('Object ID'))
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, db_constraint=False,
                                     verbose_name=_('Content type'))
    object_repr = models.TextField(null=True, blank=True, verbose_name=_('Object representation'))
    object_json_repr = models.TextField(null=True, blank=True, verbose_name=_('Object JSON representation'))

    browser = models.TextField(null=True, blank=True, verbose_name=_('Browser fields'))
    platform = models.TextField(null=True, blank=True, verbose_name=_('Platform fields'))
    operating_system = models.TextField(null=True, blank=True, verbose_name=_('Operating System fields'))

    latitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('latitude fields'))
    longitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('longitude fields'))
    city = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('city fields'))
    country = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('country fields'))

    remote_ip = models.CharField(max_length=50, null=True, db_index=True, verbose_name=_('Remote IP'))
    changed_fields = models.TextField(null=True, blank=True, verbose_name=_('Changed fields'))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True,
                             blank=True, on_delete=models.SET_NULL,
                             db_constraint=False, verbose_name=_('User'))
    user_pk_as_string = models.CharField(max_length=255, null=True, blank=True,
                                         help_text=_('String version of the user pk'),
                                         verbose_name=_('User PK as string'))
    datetime = models.DateTimeField(auto_now_add=True, verbose_name=_('Date time'))

    def is_create(self):
        return self.CREATE == self.event_type

    def is_update(self):
        return self.UPDATE == self.event_type

    def is_delete(self):
        return self.DELETE == self.event_type

    class Meta:
        verbose_name = _('CRUD event')
        verbose_name_plural = _('CRUD events')
        ordering = ['-datetime']
        index_together = ['object_id', 'content_type', ]


class LoginEvent(models.Model):
    id = models.UUIDField(primary_key=True, default=default_uuid, editable=False, unique=True)
    LOGIN = 0
    LOGOUT = 1
    FAILED = 2
    TYPES = (
        (LOGIN, _('Login')),
        (LOGOUT, _('Logout')),
        (FAILED, _('Failed login')),
    )
    login_type = models.SmallIntegerField(choices=TYPES, verbose_name=_('Event type'))
    username = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Username'))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True,
                             on_delete=models.SET_NULL, db_constraint=False,
                             verbose_name=_('User'))

    browser = models.TextField(null=True, blank=True, verbose_name=_('Browser fields'))
    platform = models.TextField(null=True, blank=True, verbose_name=_('Platform fields'))
    operating_system = models.TextField(null=True, blank=True, verbose_name=_('Operating System fields'))

    latitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('latitude fields'))
    longitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('longitude fields'))
    city = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('city fields'))
    country = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('country fields'))

    remote_ip = models.CharField(max_length=50, null=True, db_index=True, verbose_name=_('Remote IP'))
    datetime = models.DateTimeField(auto_now_add=True, verbose_name=_('Date time'))

    class Meta:
        verbose_name = _('login event')
        verbose_name_plural = _('login events')
        ordering = ['-datetime']


class RequestEvent(models.Model):
    id = models.UUIDField(primary_key=True, default=default_uuid, editable=False, unique=True)
    url = models.CharField(null=False, db_index=True, max_length=254, verbose_name=_('URL'))
    method = models.CharField(max_length=20, null=False, db_index=True, verbose_name=_('Method'))
    query_string = models.TextField(null=True, blank=True, verbose_name=_('Query string'))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True,
                             on_delete=models.SET_NULL, db_constraint=False,
                             verbose_name=_('User'))

    browser = models.TextField(null=True, blank=True, verbose_name=_('Browser fields'))
    platform = models.TextField(null=True, blank=True, verbose_name=_('Platform fields'))
    operating_system = models.TextField(null=True, blank=True, verbose_name=_('Operating System fields'))

    latitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('latitude fields'))
    longitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('longitude fields'))
    city = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('city fields'))
    country = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('country fields'))

    remote_ip = models.CharField(max_length=50, null=True, blank=True, db_index=True, verbose_name=_('Remote IP'))
    datetime = models.DateTimeField(auto_now_add=True, blank=True, db_index=True, verbose_name=_('Date time'))

    class Meta:
        verbose_name = _('request event')
        verbose_name_plural = _('request events')
        ordering = ['-datetime']


class CorsEvent(models.Model):
    id = models.UUIDField(primary_key=True, default=default_uuid, editable=False, unique=True)
    url = models.CharField(null=True, blank=True, db_index=True, max_length=254, verbose_name=_('URL'))
    method = models.CharField(max_length=20, null=True, db_index=True, verbose_name=_('Method'))
    query_string = models.TextField(null=True, verbose_name=_('Query string'))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True,
                             on_delete=models.SET_NULL, db_constraint=False,
                             verbose_name=_('User'))

    browser = models.TextField(null=True, blank=True, verbose_name=_('Browser fields'))
    platform = models.TextField(null=True, blank=True, verbose_name=_('Platform fields'))
    operating_system = models.TextField(null=True, blank=True, verbose_name=_('Operating System fields'))

    latitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('latitude fields'))
    longitude = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('longitude fields'))
    city = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('city fields'))
    country = models.CharField(max_length=500, blank=True, null=True, verbose_name=_('country fields'))

    remote_ip = models.CharField(max_length=50, null=True, db_index=True, verbose_name=_('Remote IP'))
    datetime = models.DateTimeField(auto_now_add=True, db_index=True, verbose_name=_('Date time'))

    class Meta:
        verbose_name = _('cors event')
        verbose_name_plural = _('cors events')
        ordering = ['-datetime']



import csv
import datetime
from django.contrib import admin
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponse
from django.urls import reverse
from django.utils.html import escape
from django.utils.safestring import mark_safe

from .admin_helpers import ActivityLogModelAdmin, prettify_json
from .models import CRUDEvent, LoginEvent, RequestEvent, CorsEvent
from .settings import (
    ADMIN_SHOW_AUTH_EVENTS,
    ADMIN_SHOW_MODEL_EVENTS,
    ADMIN_SHOW_REQUEST_EVENTS,
    CRUD_EVENT_LIST_FILTER,
    CRUD_EVENT_SEARCH_FIELDS,
    LOGIN_EVENT_LIST_FILTER,
    LOGIN_EVENT_SEARCH_FIELDS,
    REQUEST_EVENT_LIST_FILTER,
    REQUEST_EVENT_SEARCH_FIELDS, ADMIN_SHOW_CORS_EVENTS, CORS_EVENT_SEARCH_FIELDS, CORS_EVENT_LIST_FILTER,
)


@admin.display(description="Export to CSV")
def export_to_csv(modeladmin, request, queryset):
    """Export event audits to csv."""
    opts = modeladmin.model._meta
    response = HttpResponse(content_type="text/csv")
    response["Content-Disposition"] = f"attachment;filename={opts.verbose_name}.csv"
    writer = csv.writer(response)
    fields = [
        field
        for field in opts.get_fields()
        if not field.many_to_many and not field.one_to_many
    ]
    # Write a first row with header information
    writer.writerow([field.verbose_name for field in fields])
    # Write data rows
    for obj in queryset:
        data_row = []
        for field in fields:
            value = getattr(obj, field.name)
            if isinstance(value, datetime.datetime):
                value = value.strftime("%d/%m/%Y")
            data_row.append(value)
        writer.writerow(data_row)

    return response


# CRUD events
class CRUDEventAdmin(ActivityLogModelAdmin):
    list_display = [
        "get_event_type_display",
        "get_content_type",
        "object_id",
        "object_repr_link",
        "user_link",
        "datetime",
        "remote_ip",
    ]
    date_hierarchy = "datetime"
    list_filter = CRUD_EVENT_LIST_FILTER
    search_fields = CRUD_EVENT_SEARCH_FIELDS
    readonly_fields = [
        "user",
        "event_type",
        "object_id",
        "get_content_type",
        "object_repr",
        "object_json_repr_prettified",
        "get_user",
        "user_pk_as_string",
        "browser",
        "platform",
        "operating_system",
        "latitude",
        "longitude",
        "city",
        "country",
        "remote_ip",
        "datetime",
        "changed_fields_prettified",
    ]
    exclude = ["object_json_repr", "changed_fields"]

    def get_changelist_instance(self, *args, **kwargs):
        changelist_instance = super().get_changelist_instance(*args, **kwargs)
        content_type_ids = [obj.content_type_id for obj in changelist_instance.result_list]
        self.content_types_by_id = {
            ct.id: ct for ct in ContentType.objects.filter(id__in=content_type_ids)
        }
        return changelist_instance

    @admin.display(description="Content Type")
    def get_content_type(self, obj):
        return self.content_types_by_id[obj.content_type_id]

    @admin.display(description="User")
    def get_user(self, obj):
        return self.users_by_id.get(obj.user_id)

    @admin.display(description="object repr")
    def object_repr_link(self, obj):
        if obj.event_type == CRUDEvent.DELETE:
            html = obj.object_repr
        else:
            escaped_obj_repr = escape(obj.object_repr)
            try:
                content_type = self.get_content_type(obj)
                url = reverse(
                    f"admin:{content_type.app_label}_{content_type.model}_change",
                    args=(obj.object_id,),
                )
                html = f'<a href="{url}">{escaped_obj_repr}</a>'
            except Exception:
                html = escaped_obj_repr
        return mark_safe(html)  # noqa: S308

    @admin.display(description="object json repr")
    def object_json_repr_prettified(self, obj):
        return prettify_json(obj.object_json_repr)

    @admin.display(description="changed fields")
    def changed_fields_prettified(self, obj):
        return prettify_json(obj.changed_fields)

    actions = [export_to_csv]


# Login events
class LoginEventAdmin(ActivityLogModelAdmin):
    list_display = [
        "datetime",
        "get_login_type_display",
        "user_link",
        "get_username",
        "remote_ip",
    ]
    date_hierarchy = "datetime"
    list_filter = LOGIN_EVENT_LIST_FILTER
    search_fields = LOGIN_EVENT_SEARCH_FIELDS
    readonly_fields = [
        "username",
        "user",
        "login_type",
        "get_username",
        "get_user",
        "browser",
        "platform",
        "operating_system",
        "latitude",
        "longitude",
        "city",
        "country",
        "remote_ip",
        "datetime",
    ]

    def get_user(self, obj):
        return self.users_by_id.get(obj.user_id)

    get_user.short_description = "User"

    def get_username(self, obj):
        user = self.get_user(obj)
        return user.get_username() if user else None

    get_username.short_description = "User name"

    actions = [export_to_csv]


# Request events
class RequestEventAdmin(ActivityLogModelAdmin):
    list_display = ["datetime", "user_link", "method", "url", "remote_ip"]
    date_hierarchy = "datetime"
    list_filter = REQUEST_EVENT_LIST_FILTER
    search_fields = REQUEST_EVENT_SEARCH_FIELDS
    readonly_fields = [
        "url",
        "method",
        "query_string",
        "get_user",
        "browser",
        "platform",
        "operating_system",
        "latitude",
        "longitude",
        "city",
        "country",
        "remote_ip",
        "datetime",
    ]

    def get_user(self, obj):
        return self.users_by_id.get(obj.user_id)

    get_user.short_description = "User"

    actions = [export_to_csv]


class CorsEventAdmin(ActivityLogModelAdmin):
    list_display = ["datetime", "user_link", "method", "url", "remote_ip"]
    date_hierarchy = "datetime"
    list_filter = CORS_EVENT_LIST_FILTER
    search_fields = CORS_EVENT_SEARCH_FIELDS
    readonly_fields = [
        "url",
        "method",
        "query_string",
        "get_user",
        "browser",
        "platform",
        "operating_system",
        "latitude",
        "longitude",
        "city",
        "country",
        "remote_ip",
        "datetime",
    ]

    def get_user(self, obj):
        return self.users_by_id.get(obj.user_id)

    get_user.short_description = "User"

    actions = [export_to_csv]


if ADMIN_SHOW_MODEL_EVENTS:
    admin.site.register(CRUDEvent, CRUDEventAdmin)

if ADMIN_SHOW_AUTH_EVENTS:
    admin.site.register(LoginEvent, LoginEventAdmin)

if ADMIN_SHOW_REQUEST_EVENTS:
    admin.site.register(RequestEvent, RequestEventAdmin)

if ADMIN_SHOW_CORS_EVENTS:
    admin.site.register(CorsEvent, CorsEventAdmin)



import datetime as dt

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import NOT_PROVIDED, DateTimeField
from django.utils import timezone
from django.utils.encoding import smart_str


def get_field_value(obj, field):
    """Get the value of a given model instance field.

    :param obj: The model instance.
    :type obj: Model
    :param field: The field you want to find the value of.
    :type field: Any
    :return: The value of the field as a string.
    :rtype: str
    """
    if isinstance(field, DateTimeField):
        # DateTimeFields are timezone-aware, so we need to convert the field
        # to its naive form before we can accurately compare them for changes.
        try:
            value = field.to_python(getattr(obj, field.name, None))
            if value is not None and settings.USE_TZ and not timezone.is_naive(value):
                value = timezone.make_naive(value, timezone=dt.timezone.utc)
        except ObjectDoesNotExist:
            value = field.default if field.default is not NOT_PROVIDED else None
    else:
        try:
            value = smart_str(getattr(obj, field.name, None))
        except ObjectDoesNotExist:
            value = field.default if field.default is not NOT_PROVIDED else None

    return value


def model_delta(old_model, new_model):
    """Provide delta/difference between two models.

    :param old: The old state of the model instance.
    :type old: Model
    :param new: The new state of the model instance.
    :type new: Model
    :return: A dictionary with the names of the changed fields as keys and a
             two tuple of the old and new field values
             as value.
    :rtype: dict
    """
    delta = {}
    fields = new_model._meta.fields
    for field in fields:
        old_value = get_field_value(old_model, field)
        new_value = get_field_value(new_model, field)
        if old_value != new_value:
            delta[field.name] = [smart_str(old_value), smart_str(new_value)]

    if len(delta) == 0:
        delta = None

    return delta


def get_m2m_field_name(model, instance):
    """Find M2M field name on instance.

    Called from m2m_changed signal
    :param model: m2m_changed signal model.
    :type model: Model
    :param instance:m2m_changed signal instance.
    :type new: Model
    :return: ManyToManyField name of instance related to model.
    :rtype: str
    """
    for x in model._meta.related_objects:
        if x.related_model().__class__ == instance.__class__:
            return x.remote_field.name
    return None


def should_propagate_exceptions():
    """Whether Django Activity Log should propagate signal handler exceptions.

    :rtype: bool
    """
    return getattr(settings, "DJANGO_ACTIVITY_LOG_PROPAGATE_EXCEPTIONS", False)


from setuptools import setup

with open("readME.md", "r") as f:
    description = f.read()

setup(
    name='django-activitylog-jwt',
    version='1.0.1',
    description='Get started with django-activitylog-jwt today to bolster the security and audit capabilities of your '
                'Django application while leveraging the power of JWT authentication.',
    author='Nand Kishore',
    author_email='knand4930@gmail.com',
    license='MIT',
    url='https://github.com/knand4930/django-activitylog-jwt.git',
    packages=['activitylog'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'djangorestframework>=3.15.1',
        'djangorestframework-jwt>=1.11.0',
        'frozenlist>=1.4.1',
        'geoip2>=4.8.0',
        'idna>=3.7',
        'maxminddb>=2.6.1',
        'multidict>=6.0.5',
        'PyJWT>=1.7.1',
        'wheel>=0.43.0',
    ],
    long_description=description,
    long_description_content_type='text/markdown',
)
